--- lib/zone.ml.orig	2011-09-15 21:20:04.000000000 -0400
+++ lib/zone.ml	2011-09-15 21:20:04.000000000 -0400
@@ -385,9 +385,13 @@
     end
 
   let default () =
+    let basedir =
+      let posix = "/usr/share/zoneinfo/posix" in
+      if Sys.file_exists posix = `Yes then posix else "/usr/share/zoneinfo"
+    in
     {
       full = false;
-      basedir = "/usr/share/zoneinfo/posix";
+      basedir;
       table = String.Table.create ~size:11 ();
     }
 
@@ -419,11 +423,21 @@
         end
 end
 
-let zone_cache = Db.default ()
+module Cache : sig val get : unit -> Db.z end = struct
+  let cache = Set_once.create ()
 
-let init () = Db.fill zone_cache
+  let get () =
+    match Set_once.get cache with
+    | Some db -> db
+    | None ->
+        let db = Db.default () in
+        Set_once.set_exn cache db;
+        db
+end
+
+let init () = Db.fill (Cache.get ())
 
-let initialized_zones () = Db.initialized_zones zone_cache
+let initialized_zones () = Db.initialized_zones (Cache.get ())
 
 let of_utc_offset offset =
   assert (offset >= -24 && offset <= 24);
@@ -463,7 +477,7 @@
     | "tyo"         -> "Asia/Tokyo"
     | _             -> zone
   in
-  Db.find_or_load zone_cache zone
+  Db.find_or_load (Cache.get ()) zone
 
 let find_exn zone =
   match find zone with
